Index: src/build.c
===================================================================
--- src/build.c	(revision 3682)
+++ src/build.c	(working copy)
@@ -108,11 +108,13 @@ static struct
 widgets;
 
 
+#ifndef G_OS_WIN32
+static void build_exit_cb(GPid child_pid, gint status, gpointer user_data);
 static gboolean build_iofunc(GIOChannel *ioc, GIOCondition cond, gpointer data);
+#endif
 static gboolean build_create_shellscript(const gchar *fname, const gchar *cmd, gboolean autoclose);
 static GPid build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *dir);
 static void set_stop_button(gboolean stop);
-static void build_exit_cb(GPid child_pid, gint status, gpointer user_data);
 static void run_exit_cb(GPid child_pid, gint status, gpointer user_data);
 static void on_build_arguments_activate(GtkMenuItem *menuitem, gpointer user_data);
 
@@ -124,6 +126,8 @@ static void on_build_execute_activate(Gt
 static void on_build_next_error(GtkMenuItem *menuitem, gpointer user_data);
 static void on_build_previous_error(GtkMenuItem *menuitem, gpointer user_data);
 static void kill_process(GPid *pid);
+static void show_build_result_message(gboolean failure);
+static void process_build_output_line(const gchar *line, gint color);
 
 
 void build_finalize()
@@ -438,8 +442,13 @@ static GPid build_spawn_cmd(GeanyDocumen
 	gchar	*locale_filename;
 	gchar	*executable;
 	gchar	*tmp;
+#ifdef G_OS_WIN32
+	gchar	*output[2];
+	gint	 status, x;
+#else
 	gint     stdout_fd;
 	gint     stderr_fd;
+#endif
 
 	g_return_val_if_fail(doc != NULL, (GPid) 1);
 
@@ -491,8 +500,14 @@ static GPid build_spawn_cmd(GeanyDocumen
 	build_info.dir = g_strdup(working_dir);
 	build_info.file_type_id = FILETYPE_ID(doc->file_type);
 
-	if (! g_spawn_async_with_pipes(working_dir, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
-						NULL, NULL, &(build_info.pid), NULL, &stdout_fd, &stderr_fd, &error))
+#ifdef G_OS_WIN32
+	if (! utils_spawn_sync(working_dir, argv, NULL, G_SPAWN_SEARCH_PATH,
+			NULL, NULL, &output[0], &output[1], &status, &error))
+#else
+	if (! g_spawn_async_with_pipes(working_dir, argv, NULL,
+			G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
+			&(build_info.pid), NULL, &stdout_fd, &stderr_fd, &error))
+#endif
 	{
 		geany_debug("g_spawn_async_with_pipes() failed: %s", error->message);
 		ui_set_statusbar(TRUE, _("Process failed (%s)"), error->message);
@@ -504,6 +519,33 @@ static GPid build_spawn_cmd(GeanyDocumen
 		return (GPid) 0;
 	}
 
+#ifdef G_OS_WIN32
+	for (x = 0; x < 2; x++)
+	{
+		if (NZV(output[x]))
+		{
+			gchar **lines = g_strsplit_set(output[x], "\r\n", -1);
+			guint i, len = g_strv_length(lines);
+
+			for (i = 0; i < len; i++)
+			{
+				if (NZV(lines[i]))
+				{
+					line = lines[i];
+					while (*line != '\0')
+					{	/* replace any conrol characters in the output */
+						if (*line < 32)
+							*line = 32;
+						line++;
+					}
+					process_build_output_line(lines[i], COLOR_BLACK);
+				}
+			}
+			g_strfreev(lines);
+		}
+	}
+	show_build_result_message(status != 0);
+#else
 	if (build_info.pid > 0)
 	{
 		g_child_watch_add(build_info.pid, (GChildWatchFunc) build_exit_cb, NULL);
@@ -516,6 +558,7 @@ static GPid build_spawn_cmd(GeanyDocumen
 		TRUE, build_iofunc, GINT_TO_POINTER(0));
 	utils_set_up_io_channel(stderr_fd, G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
 		TRUE, build_iofunc, GINT_TO_POINTER(1));
+#endif
 
 	g_strfreev(argv);
 	g_free(working_dir);
@@ -762,20 +805,12 @@ static GPid build_run_cmd(GeanyDocument 
 }
 
 
-static gboolean build_iofunc(GIOChannel *ioc, GIOCondition cond, gpointer data)
-{
-	if (cond & (G_IO_IN | G_IO_PRI))
+static void process_build_output_line(const gchar *str, gint color)
 	{
-		/*GIOStatus s;*/
-		gchar *msg;
+	gchar *msg, *tmp;
 
-		while (g_io_channel_read_line(ioc, &msg, NULL, NULL, NULL) && msg)
-		{
-			/*if (s != G_IO_STATUS_NORMAL && s != G_IO_STATUS_EOF) break;*/
-			gint color;
-			gchar *tmp;
+	msg = g_strdup(str);
 
-			color = (GPOINTER_TO_INT(data)) ? COLOR_DARK_RED : COLOR_BLACK;
 			g_strchomp(msg);
 
 			if (build_parse_make_dir(msg, &tmp))
@@ -788,8 +823,7 @@ static gboolean build_iofunc(GIOChannel 
 				gchar *filename;
 				gint line;
 
-				msgwin_parse_compiler_error_line(msg, current_dir_entered,
-					&filename, &line);
+		msgwin_parse_compiler_error_line(msg, current_dir_entered, &filename, &line);
 				if (line != -1 && filename != NULL)
 				{
 					GeanyDocument *doc = document_find_by_filename(filename);
@@ -804,6 +838,21 @@ static gboolean build_iofunc(GIOChannel 
 
 			g_free(msg);
 		}
+
+
+static gboolean build_iofunc(GIOChannel *ioc, GIOCondition cond, gpointer data)
+{
+	if (cond & (G_IO_IN | G_IO_PRI))
+	{
+		gchar *msg;
+
+		while (g_io_channel_read_line(ioc, &msg, NULL, NULL, NULL) && msg)
+		{
+			gint color = (GPOINTER_TO_INT(data)) ? COLOR_DARK_RED : COLOR_BLACK;
+
+			process_build_output_line(msg, color);
+ 			g_free(msg);
+		}
 	}
 	if (cond & (G_IO_ERR | G_IO_HUP | G_IO_NVAL))
 		return FALSE;
@@ -884,7 +933,6 @@ static void show_build_result_message(gb
 
 static void build_exit_cb(GPid child_pid, gint status, gpointer user_data)
 {
-#ifdef G_OS_UNIX
 	gboolean failure = FALSE;
 
 	if (WIFEXITED(status))
@@ -902,9 +950,6 @@ static void build_exit_cb(GPid child_pid
 		failure = TRUE;
 	}
 	show_build_result_message(failure);
-#else
-	show_build_result_message(! win32_get_exit_status(child_pid));
-#endif
 
 	utils_beep();
 	g_spawn_close_pid(child_pid);
